Learn R Programming

lazysf (version 0.3.0)

lazysf-package: lazysf: Delayed Read for 'GDAL' Vector Data Sources

Description

Lazy read for drawings. A 'dplyr' back end for data sources supported by 'GDAL' vector drivers, that allows working with local or remote sources as if they are in-memory data frames. Basic features works with any drawing format ('GDAL vector data source') supported by the 'sf' package.

Arguments

Package Options

There is a debug option options(lazysf.query.debug = TRUE) which if set will cause the generated SQL statement to be printed before every call to sf::st_read(). In addition it will print the number of rows actual read.

Author

Maintainer: Michael Sumner mdsumner@gmail.com (ORCID)

See Also